



use testing

GO


-- STRING_SPLIT function
-- This function makes our life easy by splitting the values stored in single row with delimiters into seperate rows. 
-- In earlier versions, you need to write a User Defined Function with while loop to acheive this. With the help of STRING_SPLIT  
-- it becomes very very easy as shown below
 
 drop function dbo.SplitString;

CREATE FUNCTION SplitString
( 
 @Input NVARCHAR(MAX),
 @Character CHAR(1)
)
RETURNS @Output TABLE (
 Item NVARCHAR(1000)
)
AS
BEGIN
 DECLARE @StartIndex INT, @EndIndex INT

 SET @StartIndex = 1
 IF SUBSTRING(@Input, LEN(@Input) - 1, LEN(@Input)) <> @Character
 BEGIN
 SET @Input = @Input + @Character
 END

 WHILE CHARINDEX(@Character, @Input) > 0
 BEGIN
 SET @EndIndex = CHARINDEX(@Character, @Input)
 
 INSERT INTO @Output(Item)
 SELECT SUBSTRING(@Input, @StartIndex, @EndIndex - 1)
 
 SET @Input = SUBSTRING(@Input, @EndIndex + 1, LEN(@Input))
 END

 RETURN
END
GO


select * from dbo.SplitString('Operating System,Database,Office',',')

select * from STRING_SPLIT ('Operating System,Database,Office',',')

GO

-- drop table #vendors 

create table #vendors 
(
    vendor_name varchar(100),
    product_name varchar(1000)
)
insert into #vendors(vendor_name,product_name)
select 'Microsoft','Operating System,Database,Office' union all
select 'Samsung','Television,Mobile'



select
    vendor_name,value as product_name
from #vendors as t1 CROSS APPLY
(select * from STRING_SPLIT (t1.product_name,',')) as t2



/*
COMPRESS function. 
This function compresses the string values using GZIP algorithm. GZIP can be used in SQL Server and at the same time in the application layer as well 
so that it is easy to have compressed data in the backend SQL Server and DECOMPRESS it in the application layer and vice versa. 
This is primarily used to reduce the size of the data particularly varchar data.

The return type of this function is VARBINARY(MAX).
*/



create table #original_data(comment varchar(max));
GO
insert into #original_data(comment)
select replicate(cast('test' as varchar(max)),10000);
Go 1000

create table #compressed_data(comment varbinary(max));
GO
insert into #compressed_data(comment)
select compress(replicate(cast('test' as varchar(max)),10000));
Go 1000

-- Now compare the data used to store these data by executing sp_spaceused stored procedure 
exec tempdb..sp_spaceused #original_data;
exec tempdb..sp_spaceused #compressed_data;


/*DECOMPRESS function. 
This function is used to get the actual data from a compressed data. The return type is binary and you need to convert it back to varchar datatype to see the original value. 
*/

create table #compress(col varbinary(max))
insert into #compress(col)
select compress('SQL Server') as compressed_Data

select 
col as compressed_value,
cast(decompress(col) as varchar(max)) as original_value
from #compress 

-- Note : DECOMPRESS may be slower because it has to convert the compressed values into original values

/*
DROP IF EXISTS
This is new DDL statement. This is already existing in MySQL for some time and it is added now in SQL Server. 
The name is self-explanatory. You do not need to check for the existance of a table using old method of checking system tables before dropping it. 
Just use this DDL.

Suppose you want to create a table named testing and you are not sure if it is already there. If there, you want to drop and recreate it. You can do it in old versions as below
*/


-- old method 
if exists(select * from sys.objects where name='testing')
drop table testing 
create table testing(col int);

-- With the new DDL it is simplified as below

drop table if exists testing 
create table testing(col int);

-- dropping multiple tables

drop table if exists testing, testing1, testing2

-- You can also drop other objects like stored procedure, view, etc
GO

create procedure testing(@id int)
as
select @id as no

GO

drop procedure if exists testing 
 
/*
JSON function

JSON , which stands for JavaScript Object Notation, is the data format which is used as a medium of exchanging data between various systems. 
If you are a NoSQL guy, you would have known it very well than others. NoSQL databases such as Azure DocumentDB, MongoDB, etc store the data a JSON format. 
It can both store structured and unstructured data. It is like an array. So from a normalised table, you can generate Json value using JSON function

Let us explore this in the fllowing example
*/

create table #students(student_name varchar(100), country varchar(100))
insert into #students(student_name,country)
select 'Saravanana','India' union all
select 'Nilesh','USA' union all
select 'James','Canada'

-- You can create JSON data by simply using for json auto clause (very similar to FOR XML)

select * from #students for json auto

-- The result is an array where each column values are seperated by colon and each row is seperated by comma with all values are wrapped in double quotes.

-- If you want to give an element name, you can use root option

select * from #students for json auto , root ('Student')

-- Now the same JSON value starts with the element "Student"

/*
OPENJASON table valued function
While the function JSON generates the data from normalised structure into a Json value, OPENJSON does the opposite. 
If you have Json string value and want to import/show in normalised structure (rows and columns), you can use OPENJSON table valued function.

Let us explore this with the fllowing example
*/

drop table if exists #students 

create table #students(student_name varchar(100), country varchar(100))

insert into #students(student_name,country)
select * from OPENJSON(
'{"Student":[{"student_name":"Saravanana","country":"India"},{"student_name":"Nilesh","country":"USA"},{"student_name":"James","country":"Canada"}]}','$.Student')
with ([student_name] varchar(100),[country] varchar(100))

select * from #students 

-- Note that the Json data starts with the name Student so you can specify it in the OPENJSON function to identify the dataset and specify the column names with proper datatype so that the value will be transformed to rows and column

GO

-- Requires Service pack 1 

create or alter procedure testing (@i int)
as
select @i as no;

select datediff(minute,'2000-01-01','9000-01-01')

select datediff_big(minute,'2000-01-01','9000-01-01')


-- R support


Exec sp_configure  'external scripts enabled', 1  
Reconfigure  with  override  

-- Restarts the SQL Server engine

Exec sp_configure  'external scripts enabled' 

-- Make sure run_time value is 1

exec sp_execute_external_script  @language =N'R',  
@script=N'OutputDataSet<-InputDataSet',    
@input_data_1 =N'select 1 as hello'  
with result sets (([hello] int not null));  
go  


exec sp_execute_external_script  @language =N'R',  
@script=N'OutputDataSet<-InputDataSet',    
@input_data_1 =N'select ''madhivanan'' as hello'  
with result sets (([hello] varchar(100) not null));  
go  
exec sp_execute_external_script  @language =N'R',  
@script=N'OutputDataSet<-InputDataSet',    
@input_data_1 =N'select getdate() as hello'  
with result sets (([hello] datetime not null));  
go  
